@@ -238,6 +238,13 @@ set_destination_for_suggested_filename (EphyDownload *download,
   g_free (dest_dir);
   g_free (dest_name);
 
+  if (strlen (destination_filename) > NAME_MAX) {
+    g_autofree char *truncated_filename = g_utf8_substring (destination_filename, 0,
+                                                            g_utf8_strlen (destination_filename, NAME_MAX));
+    g_free (destination_filename);
+    destination_filename = g_steal_pointer (&truncated_filename);
+  }
+
   /* Append (n) as needed. */
   if (!webkit_download_get_allow_overwrite (download->download) && g_file_test (destination_filename, G_FILE_TEST_EXISTS)) {
     int i = 1;
@@ -263,6 +270,7 @@ set_destination_for_suggested_filename (EphyDownload *download,
       g_free (serial);
     } while (g_file_test (tmp_filename->str, G_FILE_TEST_EXISTS));
 
+    g_free (destination_filename);
     destination_filename = g_strdup (tmp_filename->str);
     g_string_free (tmp_filename, TRUE);
   }
@@ -431,13 +439,13 @@ ephy_download_do_download_action (EphyDownload           *download,
     case EPHY_DOWNLOAD_ACTION_BROWSE_TO:
       LOG ("ephy_download_do_download_action: browse_to");
       /* Must not use this action type under sandbox! */
-      ret = ephy_file_browse_to (destination);
+      ret = ephy_file_browse_to (destination, NULL);
       break;
     case EPHY_DOWNLOAD_ACTION_OPEN:
       LOG ("ephy_download_do_download_action: open");
-      ret = ephy_file_launch_uri_handler (destination, NULL);
+      ret = ephy_file_launch_uri_handler (destination, NULL, NULL);
       if (!ret)
-        ret = ephy_file_browse_to (destination);
+        ret = ephy_file_browse_to (destination, NULL);
       break;
     case EPHY_DOWNLOAD_ACTION_NONE:
       LOG ("ephy_download_do_download_action: none");
@@ -648,7 +656,7 @@ suggested_filename_data_free (SuggestedFilenameData *data)
 }
 
 static void
-filename_suggested_dialog_cb (AdwMessageDialog      *dialog,
+filename_suggested_dialog_cb (AdwAlertDialog        *dialog,
                               const char            *response,
                               SuggestedFilenameData *data)
 {
@@ -729,7 +737,7 @@ open_download_confirmation_dialog (EphyDownload *download,
                                    const char   *suggested_filename)
 {
   GApplication *application;
-  GtkWidget *dialog = NULL;
+  AdwDialog *dialog = NULL;
   GtkWidget *grid;
   GtkWindow *toplevel;
   GtkWidget *icon;
@@ -755,13 +763,11 @@ open_download_confirmation_dialog (EphyDownload *download,
   application = G_APPLICATION (ephy_embed_shell_get_default ());
   toplevel = gtk_application_get_active_window (GTK_APPLICATION (application));
 
-  dialog = adw_message_dialog_new (GTK_WINDOW (toplevel),
-                                   _("Download Requested"),
-                                   NULL);
-  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
-                                    "cancel", _("_Cancel"),
-                                    "download", _("_Download"),
-                                    NULL);
+  dialog = adw_alert_dialog_new (_("Download Requested"), NULL);
+  adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (dialog),
+                                  "cancel", _("_Cancel"),
+                                  "download", _("_Download"),
+                                  NULL);
 
   webkit_download = ephy_download_get_webkit_download (download);
   response = webkit_download_get_response (webkit_download);
@@ -769,7 +775,7 @@ open_download_confirmation_dialog (EphyDownload *download,
   grid = gtk_grid_new ();
   gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
   gtk_widget_set_margin_top (grid, 6);
-  adw_message_dialog_set_extra_child (ADW_MESSAGE_DIALOG (dialog), grid);
+  adw_alert_dialog_set_extra_child (ADW_ALERT_DIALOG (dialog), grid);
 
   content_length = g_format_size (webkit_uri_response_get_content_length (response));
   content_type = ephy_download_get_content_type (download);
@@ -845,7 +851,7 @@ open_download_confirmation_dialog (EphyDownload *download,
   g_signal_connect (dialog, "response",
                     G_CALLBACK (filename_suggested_dialog_cb), data);
 
-  gtk_window_present (GTK_WINDOW (dialog));
+  adw_dialog_present (dialog, GTK_WIDGET (toplevel));
 }
 
 static void
@@ -1008,7 +1014,7 @@ download_finished_cb (WebKitDownload *wk_download,
   file = g_file_new_for_path (webkit_download_get_destination (wk_download));
   download->file_monitor = g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, &error);
   if (!download->file_monitor)
-    g_warning ("Could not add a file monitor for %s, error: %s\n", g_file_get_uri (file), error->message);
+    g_warning ("Could not add a file monitor for %s, error: %s", g_file_get_uri (file), error->message);
   else
     g_signal_connect_object (download->file_monitor, "changed", G_CALLBACK (download_file_monitor_changed), download, 0);
 }
