@@ -25,6 +25,7 @@
 #include <glib.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
+#include <libportal/portal-helpers.h>
 
 extern GQuark ephy_file_helpers_error_quark;
 #define EPHY_FILE_HELPERS_ERROR_QUARK        (ephy_file_helpers_error_quark)
@@ -41,11 +42,6 @@ typedef enum
   EPHY_FILE_HELPERS_TESTING_MODE     = 1 << 5
 } EphyFileHelpersFlags;
 
-typedef enum
-{
-  EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK
-} EphyFileHelpersNotFlatpakTag;
-
 gboolean           ephy_file_helpers_init                   (const char            *profile_dir,
                                                              EphyFileHelpersFlags   flags,
                                                              GError               **error);
@@ -60,31 +56,26 @@ char       *       ephy_default_config_dir                  (void);
 void               ephy_file_helpers_shutdown               (void);
 char       *       ephy_file_get_downloads_dir              (void);
 char       *       ephy_file_desktop_dir                    (void);
+char       *       ephy_file_get_display_name               (GFile                 *file);
 const char *       ephy_file_tmp_dir                        (void);
 char       *       ephy_file_tmp_filename                   (const char            *base,
                                                              const char            *extension);
 gboolean           ephy_ensure_dir_exists                   (const char            *dir,
                                                              GError               **error);
-gboolean           ephy_file_launch_handler                 (GFile                 *file,
-                                                             guint32                user_time);
+gboolean           ephy_file_launch_uri_handler             (GFile                 *file,
+                                                             const char            *mime_type,
+                                                             GdkDisplay            *display);
 gboolean           ephy_file_delete_dir_recursively         (const char            *directory,
                                                              GError               **error);
 char       *       ephy_sanitize_filename                   (char                  *filename);
 void               ephy_open_default_instance_window        (void);
 void               ephy_open_incognito_window               (const char            *uri);
-
-/* These functions attempt to launch a particular application chosen by
- * Epiphany, which is not possible to do when running inside flatpak. Be
- * careful!
- */
-gboolean           ephy_file_launch_desktop_file            (const char                   *filename,
-                                                             guint32                       user_time,
-                                                             EphyFileHelpersNotFlatpakTag  tag);
-gboolean           ephy_file_open_uri_in_default_browser    (const char                   *uri,
-                                                             guint32                       user_time,
-                                                             GdkScreen                    *screen,
-                                                             EphyFileHelpersNotFlatpakTag  tag);
-gboolean           ephy_file_browse_to                      (GFile                        *file,
-                                                             guint32                       user_time);
+gboolean           ephy_file_open_uri_in_default_browser    (const char            *uri,
+                                                             GdkDisplay            *display);
+gboolean           ephy_file_browse_to                      (GFile                 *file,
+                                                             GdkDisplay            *display);
+void               ephy_copy_directory                      (const char            *source,
+                                                             const char            *target);
+XdpPortal  *       ephy_get_portal                          (void);
 
 G_END_DECLS
