@@ -380,8 +380,7 @@ tab_view_notify_selected_page_cb (AdwTabView  *tab_view,
 }
 
 static void
-session_maybe_open_window (EphySession *session,
-                           guint32      user_time)
+session_maybe_open_window (EphySession *session)
 {
   EphyShell *shell = ephy_shell_get_default ();
 
@@ -869,11 +868,25 @@ save_session_in_thread_finished_cb (GObject      *source_object,
 static gboolean
 session_seems_reasonable (GList *windows)
 {
+  /* The goal here is to check for a *corrupted* session. It's perfectly fine
+   * for the session to contain URLs that cannot actually be saved, including
+   * data, blob, empty, and NULL URLs.
+   *
+   * The most common indicator of session corruption is if it contains an
+   * HTTP/HTTPS URL with no host component, so that's what we're really checking
+   * for here.
+   *
+   * Of course the session is not supposed to become corrupted in the first
+   * place, but it's been known to happen if WebKit or Epiphany is seriously
+   * broken for some reason. If we have never successfully loaded any page, or
+   * any web view has a bad URL, then something has probably gone wrong inside
+   * WebKit. Do not clobber an existing good session file with our new bogus
+   * state. Bug #768250.
+   */
   for (GList *w = windows; w != NULL; w = w->next) {
     for (GList *t = ((SessionWindow *)w->data)->tabs; t != NULL; t = t->next) {
       const char *url = ((SessionTab *)t->data)->url;
       g_autoptr (GUri) uri = NULL;
-      gboolean sane = FALSE;
 
       if (!should_save_url (url))
         continue;
@@ -882,19 +895,19 @@ session_seems_reasonable (GList *windows)
       if (g_str_has_prefix (url, "about:"))
         continue;
 
-      uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, NULL);
+      uri = g_uri_parse (url,
+                         G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED,
+                         NULL);
       if (uri) {
         if (g_uri_get_host (uri) != NULL ||
             strcmp (g_uri_get_scheme (uri), "file") == 0 ||
             strcmp (g_uri_get_scheme (uri), "ephy-reader") == 0 ||
             strcmp (g_uri_get_scheme (uri), "view-source") == 0)
-          sane = TRUE;
+          continue;
       }
 
-      if (!sane) {
-        g_critical ("Refusing to save session due to invalid URL %s", url);
-        return FALSE;
-      }
+      g_critical ("Refusing to save session due to invalid URL %s", url);
+      return FALSE;
     }
   }
 
@@ -1004,20 +1017,13 @@ ephy_session_save_timeout_cb (EphySession *session)
   if (!session->loaded_page)
     return G_SOURCE_REMOVE;
 
-  /* If we have never successfully loaded any page, or any web view has an
-   * insane URL, then something has probably gone wrong inside WebKit. For
-   * instance, if the web process is nonfunctional, the UI process could have
-   * an invalid URI property. Yes, this would be a WebKit bug, but Epiphany
-   * should be robust to such issues. Do not clobber an existing good session
-   * file with our new bogus state. Bug #768250.
-   */
   data = save_data_new (session);
   if (!session_seems_reasonable (data->windows)) {
     save_data_free (data);
     return G_SOURCE_REMOVE;
   }
 
-  LOG ("ephy_sesion_save");
+  LOG ("ephy_session_save");
 
   if (ephy_shell_get_n_windows (shell) == 0) {
     session_delete (session);
@@ -1072,8 +1078,7 @@ confirm_before_recover (EphyWindow *window,
   embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
                                    title, NULL,
                                    window, NULL,
-                                   EPHY_NEW_TAB_APPEND_LAST,
-                                   0);
+                                   EPHY_NEW_TAB_APPEND_LAST);
 
   ephy_web_view_load_error_page (ephy_embed_get_web_view (embed), url,
                                  EPHY_WEB_VIEW_ERROR_PAGE_CRASH, NULL, NULL);
@@ -1081,7 +1086,6 @@ confirm_before_recover (EphyWindow *window,
 
 typedef struct {
   EphySession *session;
-  guint32 user_time;
 
   EphyWindow *window;
   gulong destroy_id;
@@ -1092,14 +1096,12 @@ typedef struct {
 } SessionParserContext;
 
 static SessionParserContext *
-session_parser_context_new (EphySession *session,
-                            guint32      user_time)
+session_parser_context_new (EphySession *session)
 {
   SessionParserContext *context;
 
   context = g_new0 (SessionParserContext, 1);
   context->session = g_object_ref (session);
-  context->user_time = user_time;
   context->is_first_window = TRUE;
 
   return context;
@@ -1247,8 +1249,7 @@ session_parse_embed (SessionParserContext  *context,
 
     embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
                                      title, NULL,
-                                     context->window, NULL, flags,
-                                     0);
+                                     context->window, NULL, flags);
 
     adw_tab_view_set_page_pinned (tab_view,
                                   adw_tab_view_get_page (tab_view, GTK_WIDGET (embed)),
@@ -1419,8 +1420,6 @@ load_stream_complete_error (GTask  *task,
                             GError *error)
 {
   EphySession *session;
-  LoadFromStreamAsyncData *data;
-  SessionParserContext *context;
 
   g_task_return_error (task, error);
 
@@ -1431,9 +1430,7 @@ load_stream_complete_error (GTask  *task,
    */
   session_delete (session);
 
-  data = g_task_get_task_data (task);
-  context = (SessionParserContext *)g_markup_parse_context_get_user_data (data->parser);
-  session_maybe_open_window (session, context->user_time);
+  session_maybe_open_window (session);
 
   g_object_unref (task);
 
@@ -1485,7 +1482,6 @@ load_stream_read_cb (GObject      *object,
  * ephy_session_load_from_stream:
  * @session: an #EphySession
  * @stream: a #GInputStream to read the session data from
- * @user_time: a user time, or 0
  * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @callback: (scope async): a #GAsyncReadyCallback to call when the
  *    request is satisfied
@@ -1501,7 +1497,6 @@ load_stream_read_cb (GObject      *object,
 void
 ephy_session_load_from_stream (EphySession         *session,
                                GInputStream        *stream,
-                               guint32              user_time,
                                GCancellable        *cancellable,
                                GAsyncReadyCallback  callback,
                                gpointer             user_data)
@@ -1524,7 +1519,7 @@ ephy_session_load_from_stream (EphySession         *session,
    */
   g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
-  context = session_parser_context_new (session, user_time);
+  context = session_parser_context_new (session);
   parser = g_markup_parse_context_new (&session_parser, 0, context, (GDestroyNotify)session_parser_context_free);
   data = load_from_stream_async_data_new (parser);
   g_task_set_task_data (task, data, (GDestroyNotify)load_from_stream_async_data_free);
@@ -1555,27 +1550,6 @@ ephy_session_load_from_stream_finish (EphySession   *session,
   return g_task_propagate_boolean (G_TASK (result), error);
 }
 
-typedef struct {
-  guint32 user_time;
-} LoadAsyncData;
-
-static LoadAsyncData *
-load_async_data_new (guint32 user_time)
-{
-  LoadAsyncData *data;
-
-  data = g_new (LoadAsyncData, 1);
-  data->user_time = user_time;
-
-  return data;
-}
-
-static void
-load_async_data_free (LoadAsyncData *data)
-{
-  g_free (data);
-}
-
 static void
 load_from_stream_cb (GObject      *object,
                      GAsyncResult *result,
@@ -1606,11 +1580,9 @@ session_read_cb (GObject      *object,
   stream = g_file_read_finish (G_FILE (object), result, &error);
   if (stream) {
     EphySession *session;
-    LoadAsyncData *data;
 
     session = EPHY_SESSION (g_task_get_source_object (task));
-    data = g_task_get_task_data (task);
-    ephy_session_load_from_stream (session, G_INPUT_STREAM (stream), data->user_time,
+    ephy_session_load_from_stream (session, G_INPUT_STREAM (stream),
                                    g_task_get_cancellable (task), load_from_stream_cb, task);
     g_object_unref (stream);
   } else {
@@ -1625,7 +1597,6 @@ session_read_cb (GObject      *object,
  * ephy_session_load:
  * @session: an #EphySession
  * @filename: the path of the source file
- * @user_time: a user time, or 0
  * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @callback: (scope async): a #GAsyncReadyCallback to call when the
  *    request is satisfied
@@ -1641,19 +1612,17 @@ session_read_cb (GObject      *object,
 void
 ephy_session_load (EphySession         *session,
                    const char          *filename,
-                   guint32              user_time,
                    GCancellable        *cancellable,
                    GAsyncReadyCallback  callback,
                    gpointer             user_data)
 {
   GFile *save_to_file;
   GTask *task;
-  LoadAsyncData *data;
 
   g_assert (EPHY_IS_SESSION (session));
   g_assert (filename);
 
-  LOG ("ephy_sesion_load %s", filename);
+  LOG ("ephy_session_load %s", filename);
 
   g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
 
@@ -1664,8 +1633,6 @@ ephy_session_load (EphySession         *session,
   g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
   save_to_file = get_session_file (filename);
-  data = load_async_data_new (user_time);
-  g_task_set_task_data (task, data, (GDestroyNotify)load_async_data_free);
   g_file_read_async (save_to_file, g_task_get_priority (task), cancellable, session_read_cb, task);
   g_object_unref (save_to_file);
 }
@@ -1727,7 +1694,6 @@ session_resumed_cb (GObject      *object,
 
 void
 ephy_session_resume (EphySession         *session,
-                     guint32              user_time,
                      GCancellable        *cancellable,
                      GAsyncReadyCallback  callback,
                      gpointer             user_data)
@@ -1749,9 +1715,9 @@ ephy_session_resume (EphySession         *session,
    * file.
    */
   if (has_session_state == FALSE) {
-    session_maybe_open_window (session, user_time);
+    session_maybe_open_window (session);
   } else if (ephy_shell_get_n_windows (shell) == 0) {
-    ephy_session_load (session, SESSION_STATE, user_time, cancellable,
+    ephy_session_load (session, SESSION_STATE, cancellable,
                        session_resumed_cb, task);
     return;
   }
