@@ -38,47 +38,24 @@
 #define HTML_STRING "testing-ephy-web-view"
 #define SERVER_PORT 12321
 
-#if SOUP_CHECK_VERSION (2, 99, 4)
 static void
 server_callback (SoupServer        *server,
                  SoupServerMessage *msg,
                  const char        *path,
                  GHashTable        *query,
                  gpointer           data)
-#else
-static void
-server_callback (SoupServer        *server,
-                 SoupMessage       *msg,
-                 const char        *path,
-                 GHashTable        *query,
-                 SoupClientContext *context,
-                 gpointer           data)
-#endif
 {
   SoupMessageHeaders *response_headers;
   SoupMessageBody *response_body;
 
-#if SOUP_CHECK_VERSION (2, 99, 4)
   response_headers = soup_server_message_get_response_headers (msg);
   response_body = soup_server_message_get_response_body (msg);
-#else
-  response_headers = msg->response_headers;
-  response_body = msg->response_body;
-#endif
 
   if (!strcmp (path, "/redirect")) {
-#if SOUP_CHECK_VERSION (2, 99, 4)
     soup_server_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY, NULL);
-#else
-    soup_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY);
-#endif
     soup_message_headers_append (response_headers, "Location", "/redirect-result");
   } else {
-#if SOUP_CHECK_VERSION (2, 99, 4)
     soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
-#else
-    soup_message_set_status (msg, SOUP_STATUS_OK);
-#endif
   }
 
 
@@ -124,34 +101,52 @@ static const URLTest test_load_url[] = {
 
   /* Require internet */
 
-  { "127.0.0.1:12321",
-    "http://127.0.0.1:12321/" },
-  { "127.0.0.1:12321/path",
-    "http://127.0.0.1:12321/path" },
+  {
+    "127.0.0.1:12321",
+    "http://127.0.0.1:12321/"
+  },
+  {
+    "127.0.0.1:12321/path",
+    "http://127.0.0.1:12321/path"
+  },
 
   /* port is SERVER_PORT */
-  { "localhost:12321",
-    "http://localhost:12321/" },
+  {
+    "localhost:12321",
+    "http://localhost:12321/"
+  },
 
 #if 0
   /* FAIL */
-  { "gnome.org:80",
-    "http://www.gnome.org/" },
+  {
+    "gnome.org:80",
+    "http://www.gnome.org/"
+  },
 #endif
 
   /* Queries */
-  { "localhost:12321/?key=value",
-    "http://localhost:12321/?key=value" },
-  { "localhost:12321/?key=value:sub-value",
-    "http://localhost:12321/?key=value:sub-value" },
-  { "localhost:12321/?key=value&key2=value2",
-    "http://localhost:12321/?key=value&key2=value2" },
-  { "localhost:12321/?key=value&key2=",
-    "http://localhost:12321/?key=value&key2=" },
+  {
+    "localhost:12321/?key=value",
+    "http://localhost:12321/?key=value"
+  },
+  {
+    "localhost:12321/?key=value:sub-value",
+    "http://localhost:12321/?key=value:sub-value"
+  },
+  {
+    "localhost:12321/?key=value&key2=value2",
+    "http://localhost:12321/?key=value&key2=value2"
+  },
+  {
+    "localhost:12321/?key=value&key2=",
+    "http://localhost:12321/?key=value&key2="
+  },
 
   /* Other HTTP status */
-  { "localhost:12321/redirect",
-    "http://localhost:12321/redirect-result" },
+  {
+    "localhost:12321/redirect",
+    "http://localhost:12321/redirect-result"
+  },
 
   /* { "about:epiphany", "ephy-about:epiphany" }, */
   /* { "about:applications", "ephy-about:applications" }, */
@@ -168,10 +163,10 @@ test_ephy_web_view_load_url (void)
     URLTest test;
     GMainLoop *loop;
     EphyWebView *view;
-    GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    GtkWidget *window = gtk_window_new ();
 
     view = EPHY_WEB_VIEW (ephy_web_view_new ());
-    gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (view));
+    gtk_window_set_child (GTK_WINDOW (window), GTK_WIDGET (view));
     test = test_load_url[i];
     loop = g_main_loop_new (NULL, FALSE);
 
@@ -187,7 +182,7 @@ test_ephy_web_view_load_url (void)
 
     g_main_loop_run (loop);
     g_main_loop_unref (loop);
-    g_object_unref (g_object_ref_sink (view));
+    g_object_unref (window);
   }
 }
 
@@ -320,37 +315,35 @@ verify_normalize_or_autosearch_urls (EphyWebView               *view,
 static void
 test_ephy_web_view_normalize_or_autosearch (void)
 {
-  char *default_engine;
   EphyWebView *view;
   EphySearchEngineManager *manager;
   EphyEmbedShell *shell;
-
+  EphySearchEngine *default_engine;
+  g_autoptr (EphySearchEngine) test_engine = NULL;
 
   view = EPHY_WEB_VIEW (ephy_web_view_new ());
 
   shell = ephy_embed_shell_get_default ();
   manager = ephy_embed_shell_get_search_engine_manager (shell);
 
-
   default_engine = ephy_search_engine_manager_get_default_engine (manager);
-  ephy_search_engine_manager_add_engine (manager,
-                                         "org.gnome.Epiphany.EphyWebViewTest",
-                                         "http://duckduckgo.com/?q=%s&t=epiphany",
-                                         "");
-  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, "org.gnome.Epiphany.EphyWebViewTest"));
+  test_engine = g_object_new (EPHY_TYPE_SEARCH_ENGINE,
+                              "name", "org.gnome.Epiphany.EphyWebViewTest",
+                              "url", "http://duckduckgo.com/?q=%s&t=epiphany",
+                              NULL);
+  ephy_search_engine_manager_add_engine (manager, test_engine);
+  ephy_search_engine_manager_set_default_engine (manager, test_engine);
+  g_assert_true (ephy_search_engine_manager_get_default_engine (manager) == test_engine);
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_ddg, G_N_ELEMENTS (normalize_or_autosearch_test_ddg));
 
-  ephy_search_engine_manager_modify_engine (manager,
-                                            "org.gnome.Epiphany.EphyWebViewTest",
-                                            "http://www.google.com/?q=%s",
-                                            "");
+  ephy_search_engine_set_url (test_engine, "http://www.google.com/?q=%s");
 
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_google, G_N_ELEMENTS (normalize_or_autosearch_test_google));
 
-  ephy_search_engine_manager_delete_engine (manager, "org.gnome.Epiphany.EphyWebViewTest");
+  ephy_search_engine_manager_delete_engine (manager, test_engine);
 
-  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, default_engine));
-  g_free (default_engine);
+  ephy_search_engine_manager_set_default_engine (manager, default_engine);
+  g_assert_true (ephy_search_engine_manager_get_default_engine (manager) == default_engine);
   g_object_unref (g_object_ref_sink (view));
 }
 
@@ -486,6 +479,9 @@ main (int   argc,
   int ret;
   SoupServer *server;
 
+  /* FIXME: disable AC mode for now for WebView tests because CI doesn't support it. */
+  g_setenv ("WEBKIT_DISABLE_COMPOSITING_MODE", "1", FALSE);
+
   gtk_test_init (&argc, &argv);
 
   ephy_debug_init ();
